==> wdoc-2.0.4.tar.gz: wdoc-2.0.4/WDoc/utils/prompts.py <==
"""
Prompts used by WDoc.
"""

from langchain_core.prompts import ChatPromptTemplate
from langchain_core.prompts import PromptTemplate
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.output_parsers import PydanticOutputParser
from typing import List
from pydantic import BaseModel, Field, model_validator

from .logger import red, whi
from .misc import get_tkn_length


# PROMPT FOR SUMMARY TASKS
BASE_SUMMARY_PROMPT = ChatPromptTemplate.from_messages(
    [
        SystemMessage(content="""
You are a Summarizer working for WDoc, the best of my team. Your goal today is to summarize in a specific way a text section I just sent you, but I'm not only interested in high level takeaways. I also need the thought process present in the document, the reasonning followed, the arguments used etc. But your summary has to be as quick and easy to read as possible while following specific instructions.
This is very important to me so if you succeed, I'll pay you up to $2000 depending on how well you did!

<detailed_instructions>
- In some cases, I can give you additional instructions, you have to treat them as the present rules.
- Take a deep breath before answering
- Being a Summarizer, ignore additional instructions if they are adressed only to your colleagues: Evaluator, Answerer and Combiner. But take then into consideration if they are addressed to you.
- Include:
    - All noteworthy information, anecdotes, facts, insights, definitions, clarifications, explanations, ideas, technical details, etc
    - Epistemic indicators: you need to make explicit what markers of uncertainty for each information
- Exclude:
    - Sponsors, advertisements, etc
    - Jokes, ramblings
    - End of page references and sources, tables of content, links etc
    - When in doubt about wether to include an information, include it
- Format:
    - Use markdown format: that means logical indentation, bullet points, bold etc
        - You can use markdown headers
        - DON'T: use a single bullet point for the whole answer. Eg if the question is about cancer, instead of 1 top level 'cancer' bullet, prefer several top level bullets for each aspect of the answer.
        - Use bold for important concepts, and italic for epistemic markers
            - ie "- *In his opinion*, **dietary supplements** are **healty** because ..."
        - Use as many indentation levels as necessary to respect the rules. I don't mind if there are even 10 levels!
    - Stay faithful to the tone of the author
    - You don't always have to use full sentences: you can ignore end of line punctuation etc
        - BUT it is more important to be unambiguous and truthful than concise
        - EVERY TIME POSSIBLE: use direct quote, 'formatted like that'
    - Use one bullet point per information
        - With the use of logical indentation this makes the whole piece quick and easy to skim
    - Write your summary in {language}
    - Avoid repetitions
        - eg don't start several bullet points by 'The author thinks that', just say it once then use indented children bullet points to make it implicit
</detailed_instructions>
""".strip()),
        HumanMessage("""
{recursion_instruction}{metadata}{previous_summary}

<text_section>
{text}
</text_section>
""".strip())
    ],
)

# if the text to summarize is long, give the end of the previous summary to help with transitions
PREV_SUMMARY_TEMPLATE = """

<end_of_summary_of_prev_section>
{previous_summary}
</end_of_summary_of_prev_section>
"""

# if the summary is recursive, add those instructions
RECURSION_INSTRUCTION = """
<additional_instructions>
I'm giving you back your own summary from last time because it was too long and contained repetitions. I want you to rewrite it as closely as possible while removing repetitions and fixing the logical indentation. Of course you have to remove the 'Chunk' indicator if present, to curate the logical indentation. You can reorganize the text freely as long as you don't lose relevant information and follow the instructions I gave you before and right now. This is important.
</additional_instructions>
""".lstrip()

# RAG
PR_EVALUATE_DOC = ChatPromptTemplate.from_messages(
    [
        SystemMessage(content="""
You are an Evaluator working for WDoc: given a question and text document. Your goal is to answer a number between 0 and 10 depending on how much the document is relevant to the question. 0 means completely irrelevant, 10 means totally relevant, and in betweens for subjective relation. If you are really unsure, you should answer '5'.

<rules>
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, double check your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer></answer> XML tag.
- The <answer> XML tag should only contain a number and nothing else.
- If the document refers to an image, take a reasonnable guess as to wether this image is probably relevant or not, even if you can't see the image.
- Being an Evaluator, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Answerer and Combiner. But take then into consideration if they are addressed to you.
</rules>
""".strip()),
        HumanMessage(content="""
<text_document>
{doc}
</text_document>

<question>
{q}
</question>

Take a deep breath.
You can start your reply when you are ready.
""".strip())
    ]
)

PR_ANSWER_ONE_DOC = ChatPromptTemplate.from_messages(
    [
        SystemMessage(content="""
You are an Answerer working for WDoc: given a piece of document and a question, your goal is to extract the relevant information while following specific instructions.

<detailed_instructions>
- If the document is ENTIRELY irrelevant to the question, answer only `<answer>IRRELEVANT</answer>` and NOTHING ELSE (and no formatting).
- Being an Answerer, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Evaluator and Combiner. But take then into consideration if they are addressed to you.
- Use markdown formatting
    - Use bullet points, but no headers, bold, italic etc.
    - Use logic based indentation for the bullet points.
    - DON'T wrap your answer in a code block or anything like that.
- Use a maximum of 5 markdown bullet points to answer the question.
    - Your answer ALWAYS HAS TO BE standalone / contextualized (i.e. both the question and its answer must be part of your reply).
    - EVERY TIME POSSIBLE: supplement your reply with direct quotes from the document.
        - Use children bullet for the quotes, between 'quotation' signs.
    - Remain concise, you can use [...] in your quotes to remove unecessary text.
- NEVER use your own knowledge of the subject, only use the document or answer `<answer>IRRELEVANT</answer>`.
- DON'T interpret the question too strictly:
    - eg: if the question is phrased as an instruction like "give me all information about such and such", use common sense and satisfy the instruction!
- ALWAYS double check that you are not contradicting the original document before answering.
- If you're unsure but the document refers to an image that has a reasonnable chance to be relevant, treat this document as if it was probably relevant.
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, recheck your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer> XML tag.
- The <answer> XML tag should only contain your answer.
</detailed_instructions>
""".strip()),
        HumanMessage(content="""
<context>
{context}
</context>

<question>
{question_to_answer}
</question>

Now take a deep breath.
Take your time.
Start your reply when you're ready.
""".strip())
    ]
)

PR_COMBINE_INTERMEDIATE_ANSWERS = ChatPromptTemplate.from_messages(
    [
        SystemMessage(content="""
You are a Combiner working for WDoc: given a question and candidate intermediate answers, your goal is to:
- combine all partial answers to answer the question as md bullet points,
- while combining all additional information as additional bullet points.
- And keeping track of sources.

<detailed_instructions>
- Being a Combiner, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Evaluator and Answerer. But take then into consideration if they are addressed to you.
- Format:
    - Use markdown format, with bullet points.
      - IMPORTANT: use logical indentation to organize information hierarchically.
        - Use as many indentation levels as necessary to respect the rules. I don't mind if there are even 10 levels!
      - The present instructions are a good example of proper formatting.
    - You are allowed to use markdown tables if appropriate.
    - Reuse acronyms without specifying what they mean.
    - Be concise but don't omit only irrelevant information from the statements.
    - Answer in the same language as the question.
- What to include:
    - If an answer refers to an image that has a reasonnable chance of being relevant, treat that as possibly relevant information.
    - Only use information from the provided statements.
        - IMPORTANT: if the statements are insufficient to answer the question you MUST start your answer by: 'OPINION:' followed by your own answer.
            - This way I know the source is you!
    - Ignore statements that are completely irrelevant to the question.
    - Semi relevant statements can be included, especially if related to possible followup questions.
    - No redundant information must remain.
        - eg: fix redundancies with one parent bullet point and several indented children.
    - DON'T interpret the question too strictly:
        - eg: if the question makes reference to "documents" consider that it's what I call here "statements" for example.
        - eg: if the question is phrased as an instruction like "give me all information about such and such", use common sense and satisfy the instruction!
- The intermediate answer can consist of a succession of thoughts in <thinking> XML tag followd by the answer in an <answer> XML tag. In that case you have to only take into account the <answer> (the <thinking> can still be helpful but don't treat it as source you can include in your own answer, you can't!)
- If some information are imcompatible, don't make a judgement call: just include both and add short clarification between parentheses and I'll take a look.
- Sources are designated by unique identifiers. Use the format [id1, id2], to keep track of each source so that we can find the original source of each information in your final answer.
    - Ideally, the sources are mentionned as close as possible to the key information, and always at the end of the bullet point.
    - It is extremely important that you do not forget to include a source.
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, recheck your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer> XML tag.
- The <answer> XML tag should only contain your answer.
</detailed_instructions>
""".strip()),
         HumanMessage(content="""
<question>
{question}
</question>

<candidate intermediate answers>
{intermediate_answers}
</candidate intermediate answers>

Now take a deep breath.
Take your time.
Start your reply when you're ready.
""".strip())
    ]
)

# embedding queries

# https://python.langchain.com/docs/how_to/output_parser_structured/
class ExpandedQuery(BaseModel):
    thoughts: str = Field(description="Reasonning to expand the query")
    output_queries: List[str] = Field(description="List containing each output query")

    @model_validator(mode="before")
    @classmethod
    def nonempty_queries(cls, values: dict) -> dict:
        oq = values["output_queries"]
        if not isinstance(oq, list):
            raise ValueError("output_queries has to be a list")
        if not oq:
            raise ValueError("output_queries can't be empty")
        if not all(isinstance(q, str) for q in oq):
            raise ValueError("output_queries has to be a list of str")
        oq = [q.strip() for q in oq if q.strip()]
        if not oq:
            raise ValueError("output_queries can't be empty after removing empty strings")
        return values

multiquery_parser = PydanticOutputParser(pydantic_object=ExpandedQuery)
PR_MULTI_QUERY_PROMPT = ChatPromptTemplate.from_messages(
    [
        SystemMessage(content="""
You are an AI language model assistant. Your are given a user
RAG query. Your task is to expand the query, meaning you have to
generate 10 different versions of the query to increase the chances of
retrieving the most relevant documents using a vector embedding search.
You can generate multiple perspectives as well as anticipate the actual
answer (like HyDE style search).
You will be given specific formatting your for your answer in due time.
I know this is a complex task so you are encouraged to express your
thinking process BEFORE answering (as long as you respect the format).

Example: if you are given a short query "breast cancer", you should expand
the query to a list of 10 similar query like "breast cancer treatment",
"diagnostics of breast cancer", "epidemiology of breast cancer", "clinical
presentation of breast cancers", "classification of breast cancer", etc.
You can also anticipate the answer like "the most used chemotherapies
for breast cancers are anthracyclines, taxanes and cyclophosphamide".
""".strip() + "\n" + multiquery_parser.get_format_instructions()),
        HumanMessage(content="Here's the query to expand: '''{question}'''"""),
    ]
)

class Prompts_class:
    evaluate: ChatPromptTemplate
    answer: ChatPromptTemplate
    combine: ChatPromptTemplate
    multiquery: PromptTemplate

    def __init__(
        self,
        **prompts: dict[str, ChatPromptTemplate],
    ) -> None:
        for key, pr in prompts.items():
            setattr(self, key, pr)

    def enable_prompt_caching(self, prompt_key: str) -> None:
        assert prompt_key in ["evaluate", "answer", "combine", "multiquery"], "Unexpected prompt_key"

        whi(f"Detected anthropic in llm name so enabling anthropic prompt_caching for {prompt_key} prompt")
        prompt = getattr(self, prompt_key)
        sys = prompt.messages[0]
        assert isinstance(sys, SystemMessage), sys
        content = sys.content
        #
        # anthropic caching works only above 1024 tokens but doesn't count
        # exactly like openai
        tkl = get_tkn_length(content)
        if tkl < 700:
            red(f"System prompt is only {tkl} openai tokens so caching will probably not work.")

        new_content = [
            {
                "type": "text",
                "text": content,
                "cache_control": {
                    "type": "ephemeral"
                }
            }
        ]
        prompt.messages[0].content = new_content

prompts = Prompts_class(
    evaluate=PR_EVALUATE_DOC,
    answer=PR_ANSWER_ONE_DOC,
    combine=PR_COMBINE_INTERMEDIATE_ANSWERS,
    multiquery=PR_MULTI_QUERY_PROMPT
)

==> wdoc-2.1.4.tar.gz: wdoc-2.1.4/WDoc/utils/prompts.py <==
"""
Prompts used by WDoc.
"""

from langchain_core.prompts import ChatPromptTemplate
from langchain_core.prompts import PromptTemplate
from langchain_core.messages import HumanMessage, SystemMessage
from langchain_core.prompts.chat import HumanMessagePromptTemplate, SystemMessagePromptTemplate
from langchain_core.output_parsers import PydanticOutputParser
from typing import List
from pydantic import BaseModel, Field, model_validator

from .logger import red, whi
from .misc import get_tkn_length


# PROMPT FOR SUMMARY TASKS
BASE_SUMMARY_PROMPT = ChatPromptTemplate(
    messages=[
        SystemMessagePromptTemplate.from_template("""
You are a Summarizer working for WDoc, the best of my team. Your goal today is to summarize in a specific way a text section I just sent you, but I'm not only interested in high level takeaways. I also need the thought process present in the document, the reasonning followed, the arguments used etc. But your summary has to be as quick and easy to read as possible while following specific instructions.
This is very important to me so if you succeed, I'll pay you up to $2000 depending on how well you did!

<detailed_instructions>
- In some cases, I can give you additional instructions, you have to treat them as the present rules.
- Take a deep breath before answering
- Being a Summarizer, ignore additional instructions if they are adressed only to your colleagues: Evaluator, Answerer and Combiner. But take then into consideration if they are addressed to you.
- Include:
    - All noteworthy information, anecdotes, facts, insights, definitions, clarifications, explanations, ideas, technical details, etc
    - Epistemic indicators: you need to make explicit what markers of uncertainty for each information
- Exclude:
    - Sponsors, advertisements, etc
    - Jokes, ramblings
    - End of page references and sources, tables of content, links etc
    - When in doubt about wether to include an information, include it
- Format:
    - Use markdown format: that means logical indentation, bullet points, bold etc
        - DON'T use markdown headers, only bullet points.
        - DON'T: use a single bullet point for the whole answer. Eg if the question is about cancer, instead of 1 top level 'cancer' bullet, prefer several top level bullets for each aspect of the answer.
        - Use bold for important concepts, and italic for epistemic markers
            - ie "- *In his opinion*, **dietary supplements** are **healty** because ..."
        - Use as many indentation levels as necessary to respect the rules. I don't mind if there are even 10 levels!
    - Stay faithful to the tone of the author
    - You don't always have to use full sentences: you can ignore end of line punctuation etc
        - BUT it is more important to be unambiguous and truthful than concise
        - EVERY TIME POSSIBLE: use direct quote, 'formatted like that'
    - Use one bullet point per information
        - With the use of logical indentation this makes the whole piece quick and easy to skim
    - Write your summary in {language}
    - Avoid repetitions
        - eg don't start several bullet points by 'The author thinks that', just say it once then use indented children bullet points to make it implicit
</detailed_instructions>
""".strip()),
        HumanMessagePromptTemplate.from_template("""
{recursion_instruction}{metadata}{previous_summary}

<text_section>
{text}
</text_section>
""".strip())
    ],
    input_variables=[
        "language",
        "recursion_instruction",
        "metadata",
        "previous_summary",
        "text",
    ],
    validate_template=True,
)

# if the text to summarize is long, give the end of the previous summary to help with transitions
PREV_SUMMARY_TEMPLATE = """

<end_of_summary_of_prev_section>
{previous_summary}
</end_of_summary_of_prev_section>
"""

# if the summary is recursive, add those instructions
RECURSION_INSTRUCTION = """
<additional_instructions>
I'm giving you back your own summary from last time because it was too long and contained repetitions. I want you to rewrite it as closely as possible while removing repetitions and fixing the logical indentation. Of course you have to remove the 'Chunk' indicator if present, to curate the logical indentation. You can reorganize the text freely as long as you don't lose relevant information and follow the instructions I gave you before and right now. This is important.
</additional_instructions>
""".lstrip()

# RAG
PR_EVALUATE_DOC = ChatPromptTemplate(
    messages=[
        SystemMessagePromptTemplate.from_template(template="""
You are an Evaluator working for WDoc: given a question and text document. Your goal is to answer a number between 0 and 10 depending on how much the document is relevant to the question. 0 means completely irrelevant, 10 means totally relevant, and in betweens for subjective relation. If you are really unsure, you should answer '5'.

<rules>
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, double check your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer></answer> XML tag.
- The <answer> XML tag should only contain a number and nothing else.
- If the document refers to an image, take a reasonnable guess as to wether this image is probably relevant or not, even if you can't see the image.
- Being an Evaluator, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Answerer and Combiner. But take then into consideration if they are addressed to you.
</rules>
""".strip()),
        HumanMessagePromptTemplate.from_template(template="""
<text_document>
{doc}
</text_document>

<question>
{q}
</question>

Take a deep breath.
You can start your reply when you are ready.
""".strip())
    ],
    validate_template=True,
    input_variables=["doc", "q"],
)

PR_ANSWER_ONE_DOC = ChatPromptTemplate(
    messages=[
        SystemMessagePromptTemplate.from_template(template="""
You are an Answerer working for WDoc: given a piece of document and a question, your goal is to extract the relevant information while following specific instructions.

<detailed_instructions>
- If the document is ENTIRELY irrelevant to the question, answer only `<answer>IRRELEVANT</answer>` and NOTHING ELSE (and no formatting).
- Being an Answerer, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Evaluator and Combiner. But take then into consideration if they are addressed to you.
- Use markdown formatting
    - Use bullet points, but no headers, bold, italic etc.
    - Use logic based indentation for the bullet points.
    - DON'T wrap your answer in a code block or anything like that.
- Use a maximum of 5 markdown bullet points to answer the question.
    - Your answer ALWAYS HAS TO BE standalone / contextualized (i.e. both the question and its answer must be part of your reply).
    - EVERY TIME POSSIBLE: supplement your reply with direct quotes from the document.
        - Use children bullet for the quotes, between 'quotation' signs.
    - Remain concise, you can use [...] in your quotes to remove unecessary text.
- NEVER use your own knowledge of the subject, only use the document or answer `<answer>IRRELEVANT</answer>`.
- DON'T interpret the question too strictly:
    - eg: if the question is phrased as an instruction like "give me all information about such and such", use common sense and satisfy the instruction!
- ALWAYS double check that you are not contradicting the original document before answering.
- If you're unsure but the document refers to an image that has a reasonnable chance to be relevant, treat this document as if it was probably relevant.
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, recheck your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer> XML tag.
- The <answer> XML tag should only contain your answer.
</detailed_instructions>
""".strip()),
        HumanMessagePromptTemplate.from_template(template="""
<context>
{context}
</context>

<question>
{question_to_answer}
</question>

Now take a deep breath.
Take your time.
Start your reply when you're ready.
""".strip())
    ],
    validate_template=True,
    input_variables=["context", "question_to_answer"],
)

PR_COMBINE_INTERMEDIATE_ANSWERS = ChatPromptTemplate(
    messages=[
        SystemMessagePromptTemplate.from_template(template="""
You are a Combiner working for WDoc: given a question and candidate intermediate answers, your goal is to:
- combine all partial answers to answer the question as md bullet points,
- while combining all additional information as additional bullet points.
- And keeping track of sources.

<detailed_instructions>
- Being a Combiner, ignore additional instructions if they are adressed only to your colleagues: Summarizer, Evaluator and Answerer. But take then into consideration if they are addressed to you.
- Format:
    - Use markdown format, with bullet points.
      - IMPORTANT: use logical indentation to organize information hierarchically.
        - Use as many indentation levels as necessary to respect the rules. I don't mind if there are even 10 levels!
      - The present instructions are a good example of proper formatting.
    - You are allowed to use markdown tables if appropriate.
    - Reuse acronyms without specifying what they mean.
    - Be concise but don't omit only irrelevant information from the statements.
    - Answer in the same language as the question.
- What to include:
    - If an answer refers to an image that has a reasonnable chance of being relevant, treat that as possibly relevant information.
    - Ignore statements that are completely irrelevant to the question.
    - Semi relevant statements can be included, especially if related to possible followup questions.
    - No redundant information must remain.
        - eg: fix redundancies with one parent bullet point and several indented children.
    - DON'T interpret the question too strictly:
        - eg: if the question makes reference to "documents" consider that it's what I call here "statements" for example.
        - eg: if the question is phrased as an instruction like "give me all information about such and such", use common sense and satisfy the instruction!
- The intermediate answer can consist of a succession of thoughts in <thinking> XML tag followd by the answer in an <answer> XML tag. In that case you have to only take into account the <answer> (the <thinking> can still be helpful but don't treat it as source you can include in your own answer, you can't!)
- If some information are imcompatible, don't make a judgement call: just include both and add short clarification between parentheses and I'll take a look.
- Sources are designated by unique identifiers. Use the format [id1, id2], to keep track of each source so that we can find the original source of each information in your final answer.
    - Ideally, the sources are mentionned as close as possible to the key information, and always at the end of the bullet point.
    - It is extremely important that you do not forget to include a source.
- Only use information from the provided statements.
    - IMPORTANT: if the statements are insufficient to answer the question you MUST specify as source [OPINION] (i.e. 'OPINION' is a valid source id). This is important, it allows me to know that the source was you for a specific information.
- Before answering, you have to think for as long as you want inside a <thinking> XML tag, then you must take a DEEP breath, recheck your answer by reasoning step by step one last time, and finally answer.
- wrap your answer in an <answer> XML tag.
- The <answer> XML tag should only contain your answer.
</detailed_instructions>
""".strip()),
         HumanMessagePromptTemplate.from_template(template="""
<question>
{question}
</question>

<candidate intermediate answers>
{intermediate_answers}
</candidate intermediate answers>

Now take a deep breath.
Take your time.
Start your reply when you're ready.
""".strip())
    ],
    validate_template=True,
    input_variables=["question", "intermediate_answers"],
)

# embedding queries

# https://python.langchain.com/docs/how_to/output_parser_structured/
class ExpandedQuery(BaseModel):
    thoughts: str = Field(description="Reasoning you followed")
    output_queries: List[str] = Field(description="List containing each output query")

    @model_validator(mode="before")
    @classmethod
    def nonempty_queries(cls, values: dict) -> dict:
        oq = values["output_queries"]
        if not isinstance(oq, list):
            raise ValueError("output_queries has to be a list")
        if not oq:
            raise ValueError("output_queries can't be empty")
        if not all(isinstance(q, str) for q in oq):
            raise ValueError("output_queries has to be a list of str")
        oq = [q.strip() for q in oq if q.strip()]
        if not oq:
            raise ValueError("output_queries can't be empty after removing empty strings")
        return values

    @model_validator(mode="after")
    @classmethod
    def remove_thoughts(cls, values) -> List[str]:
        return values.output_queries

multiquery_parser = PydanticOutputParser(pydantic_object=ExpandedQuery)
PR_MULTI_QUERY_PROMPT = ChatPromptTemplate(
    messages=[
        # has to be a systemmessage directly otherwise pydantic's fstrings impact the validation
        SystemMessage(content="""
You are an AI language model assistant. Your are given a user
RAG query. Your task is to expand the query, meaning you have to
generate 10 different versions of the query to increase the chances of
retrieving the most relevant documents using a vector embedding search.
You can generate multiple perspectives as well as anticipate the actual
answer (like HyDE style search).
You will be given specific formatting your for your answer in due time.
I know this is a complex task so you are encouraged to express your
thinking process BEFORE answering (as long as you respect the format).

Example: if you are given a short query "breast cancer", you should expand
the query to a list of 10 similar query like "breast cancer treatment",
"diagnostics of breast cancer", "epidemiology of breast cancer", "clinical
presentation of breast cancers", "classification of breast cancer", etc.
You can also anticipate the answer like "the most used chemotherapies
for breast cancers are anthracyclines, taxanes and cyclophosphamide".

DO NOT reply anything that is not either a thought or the queries. You
have to do your thinking INSIDE the thought json.
""".strip()+ "\n" + multiquery_parser.get_format_instructions()),
        HumanMessagePromptTemplate.from_template(
            template="Here's the query to expand: '''{question}'''"""),
    ],
    input_variables=["question"],
    validate_template=True,
)

class Prompts_class:
    evaluate: ChatPromptTemplate
    answer: ChatPromptTemplate
    combine: ChatPromptTemplate
    multiquery: PromptTemplate

    def __init__(
        self,
        **prompts: dict[str, ChatPromptTemplate],
    ) -> None:
        for key, pr in prompts.items():
            setattr(self, key, pr)

    def enable_prompt_caching(self, prompt_key: str) -> None:
        assert prompt_key in ["evaluate", "answer", "combine", "multiquery"], "Unexpected prompt_key"

        whi(f"Detected anthropic in llm name so enabling anthropic prompt_caching for {prompt_key} prompt")
        prompt = getattr(self, prompt_key)
        sys = prompt.messages[0]
        assert isinstance(sys, (SystemMessagePromptTemplate, SystemMessage)), sys
        if hasattr(sys, "content"):
            content = sys.content
        else:
            content = sys.prompt.template
        #
        # anthropic caching works only above 1024 tokens but doesn't count
        # exactly like openai
        tkl = get_tkn_length(content)
        if tkl < 750:
            red(f"System prompt is only {tkl} openai tokens so caching will won't work, not using it.")
            return

        new_content = [
            {
                "type": "text",
                "text": content,
                "cache_control": {
                    "type": "ephemeral"
                }
            }
        ]
        if hasattr(sys, "content"):
            prompt.messages[0].content = new_content
        else:
            prompt.messages[0].prompt.template = new_content

prompts = Prompts_class(
    evaluate=PR_EVALUATE_DOC,
    answer=PR_ANSWER_ONE_DOC,
    combine=PR_COMBINE_INTERMEDIATE_ANSWERS,
    multiquery=PR_MULTI_QUERY_PROMPT
)
